"""
Configuration settings for the LLM Document Analysis package.
"""
from enum import Enum
from typing import Optional
from pydantic import Field

from pydantic_settings import BaseSettings, SettingsConfigDict


class LLMProvider(str, Enum):
    """Supported LLM providers."""
    
    OLLAMA = "ollama"
    OPENAI = "openai"


class Settings(BaseSettings):
    """Main settings for the LLM Document Analysis package."""
    
    # LLM Configuration
    llm_provider: LLMProvider = LLMProvider.OLLAMA
    ollama_base_url: str = Field(
        default="http://localhost:11434",
        description="Base URL for Ollama API"
    )
    ollama_model: str = Field(
        default="deepseek-r1:32b",
        description="Name of the Ollama model to use"
    )
    openai_api_key: Optional[str] = None
    model_name: str = Field(
        default="deepseek-r1:32b",
        description="Name of the LLM model to use"
    )
    temperature: float = 0.0
    max_tokens: int = 1000
    
    # Document Processing
    chunk_size: int = 1000
    chunk_overlap: int = 200
    max_doc_size_mb: float = 10.0
    
    # Logging
    log_level: str = "INFO"
    
    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        case_sensitive=False,
    )
    
    @property
    def is_ollama(self) -> bool:
        """Check if using Ollama provider."""
        return self.llm_provider == LLMProvider.OLLAMA
    
    @property
    def is_openai(self) -> bool:
        """Check if using OpenAI provider."""
        return self.llm_provider == LLMProvider.OPENAI


settings = Settings() 