"""
Core document analyzer implementation using LangChain.
"""
from typing import Any, Dict, List, Optional, Union

from langchain.prompts import ChatPromptTemplate
from langchain_core.language_models.base import BaseLanguageModel
from langchain_ollama import OllamaLLM
from langchain_openai import ChatOpenAI

from ..config.settings import LLMProvider, settings
from ..models.document import Document, DocumentChunk


class DocumentAnalyzer:
    """Main document analyzer class using LLM capabilities."""
    
    def __init__(
        self,
        model_name: Optional[str] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        llm_provider: Optional[Union[LLMProvider, str]] = None,
    ):
        """Initialize the document analyzer.
        
        Args:
            model_name: Name of the LLM model to use
            temperature: Model temperature (0-1)
            max_tokens: Maximum tokens for model response
            llm_provider: LLM provider to use (Ollama or OpenAI)
            
        Raises:
            ValueError: If an invalid LLM provider is specified
        """
        self.model_name = model_name or settings.model_name
        self.temperature = temperature or settings.temperature
        self.max_tokens = max_tokens or settings.max_tokens
        
        if isinstance(llm_provider, str):
            try:
                llm_provider = LLMProvider(llm_provider)
            except ValueError:
                raise ValueError(f"Invalid LLM provider: {llm_provider}. Must be one of {[p.value for p in LLMProvider]}")
        
        self.llm_provider = llm_provider or settings.llm_provider
        self.llm = self._initialize_llm()
    
    def _initialize_llm(self) -> BaseLanguageModel:
        """Initialize the appropriate LLM based on provider.
        
        Returns:
            BaseLanguageModel: Initialized language model
            
        Raises:
            ValueError: If an invalid LLM provider is specified
        """
        if self.llm_provider == LLMProvider.OLLAMA:
            return OllamaLLM(
                model=self.model_name,
                temperature=self.temperature,
                base_url=settings.ollama_base_url,
            )
        elif self.llm_provider == LLMProvider.OPENAI:
            return ChatOpenAI(
                model_name=self.model_name,
                temperature=self.temperature,
                max_tokens=self.max_tokens,
            )
        else:
            raise ValueError(f"Invalid LLM provider: {self.llm_provider}")
    
    def analyze_document(
        self,
        document: Document,
        analysis_prompt: str,
        *,
        per_chunk: bool = False
    ) -> Union[str, List[str]]:
        """Analyze a document using the specified prompt.
        
        Args:
            document: Document to analyze
            analysis_prompt: Prompt template for analysis
            per_chunk: Whether to analyze each chunk separately
            
        Returns:
            Union[str, List[str]]: Analysis result(s)
        """
        if per_chunk:
            return self._analyze_chunks(document.chunks, analysis_prompt)
        return self._analyze_full_document(document, analysis_prompt)
    
    def _analyze_full_document(self, document: Document, prompt_template: str) -> str:
        """Analyze the entire document at once.
        
        Args:
            document: Document to analyze
            prompt_template: Prompt template for analysis
            
        Returns:
            str: Analysis result
        """
        prompt = ChatPromptTemplate.from_template(prompt_template)
        chain = prompt | self.llm
        
        result = chain.invoke({
            "text": document.content,
            "metadata": document.metadata.model_dump(),
        })
        
        return result.content if hasattr(result, 'content') else str(result)
    
    def _analyze_chunks(
        self,
        chunks: List[DocumentChunk],
        prompt_template: str
    ) -> List[str]:
        """Analyze document chunks separately.
        
        Args:
            chunks: List of document chunks to analyze
            prompt_template: Prompt template for analysis
            
        Returns:
            List[str]: Analysis results for each chunk
        """
        prompt = ChatPromptTemplate.from_template(prompt_template)
        chain = prompt | self.llm
        
        results = []
        for chunk in chunks:
            result = chain.invoke({
                "text": chunk.text,
                "metadata": chunk.metadata,
            })
            results.append(result.content if hasattr(result, 'content') else str(result))
        
        return results
    
    def summarize(self, document: Document, max_length: int = 1000) -> str:
        """Generate a summary of the document.
        
        Args:
            document: Document to summarize
            max_length: Maximum length of the summary
            
        Returns:
            str: Document summary
            
        Raises:
            ValueError: If max_length is less than or equal to 0
        """
        if max_length <= 0:
            raise ValueError("max_length must be greater than 0")
            
        prompt_template = """
        You are a professional document summarizer. Your task is to provide a concise summary of the following text.

        Requirements:
        1. Maximum length: {max_length} characters
        2. Focus on main points and key information
        3. Be direct and concise
        4. Do not include any meta-commentary or thinking process
        5. Respond with ONLY the summary text, nothing else

        Text to summarize:
        {text}

        Summary (max {max_length} characters):
        """
        
        prompt = ChatPromptTemplate.from_template(prompt_template)
        chain = prompt | self.llm
        
        result = chain.invoke({
            "text": document.content,
            "max_length": max_length,
        })
        
        # Extract the actual summary text from the response
        response = result.content if hasattr(result, 'content') else str(result)
        
        # Remove any meta-commentary or thinking process
        if '<think>' in response and '</think>' in response:
            response = response.split('</think>')[-1].strip()
        
        # Ensure the summary is within the length limit
        if len(response) > max_length:
            response = response[:max_length].rsplit(' ', 1)[0].strip()
        
        return response
    
    def extract_key_information(
        self,
        document: Document,
        info_types: List[str]
    ) -> Dict[str, Any]:
        """Extract specific types of information from the document.
        
        Args:
            document: Document to analyze
            info_types: List of information types to extract
            
        Returns:
            Dict[str, Any]: Extracted information
            
        Raises:
            ValueError: If info_types is empty
        """
        if not info_types:
            raise ValueError("info_types cannot be empty")
            
        prompt_template = """
        Please extract the following types of information from the text:
        {info_types}
        
        Format the response as a JSON object with the information types as keys.
        
        Text: {text}
        
        Extracted Information:
        """
        
        prompt = ChatPromptTemplate.from_template(prompt_template)
        chain = prompt | self.llm
        
        result = chain.invoke({
            "text": document.content,
            "info_types": "\n".join(f"- {info_type}" for info_type in info_types),
        })
        
        return result.content if hasattr(result, 'content') else str(result) 