"""
Core document models for the LLM Document Analysis package.
"""
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field, ConfigDict


class DocumentType(str, Enum):
    """Supported document types."""
    
    PDF = "pdf"
    DOCX = "docx"
    TXT = "txt"
    XLSX = "xlsx"
    HTML = "html"
    UNKNOWN = "unknown"


class DocumentMetadata(BaseModel):
    """Metadata for a document."""
    
    filename: str
    file_type: DocumentType
    file_size: int
    created_at: datetime = Field(default_factory=datetime.now)
    modified_at: datetime = Field(default_factory=datetime.now)
    num_pages: Optional[int] = None
    author: Optional[str] = None
    title: Optional[str] = None
    custom_metadata: Dict[str, Any] = Field(default_factory=dict)


class DocumentChunk(BaseModel):
    """A chunk of text from a document."""
    
    text: str
    page_number: Optional[int] = None
    chunk_index: int
    metadata: Dict[str, Any] = Field(default_factory=dict)


class Document(BaseModel):
    """Main document class containing content and metadata."""
    
    model_config = ConfigDict(arbitrary_types_allowed=True)
    
    content: str
    metadata: DocumentMetadata
    chunks: List[DocumentChunk] = Field(default_factory=list)
    analysis_results: Dict[str, Any] = Field(default_factory=dict)
    
    def to_dict(self) -> Dict:
        """Convert document to dictionary format."""
        return self.model_dump()
    
    @property
    def num_chunks(self) -> int:
        """Get the number of chunks in the document."""
        return len(self.chunks) 