"""
Base document processor implementation.
"""
from abc import ABC, abstractmethod
from typing import List, Optional

from ..models.document import Document, DocumentChunk, DocumentMetadata, DocumentType


class BaseDocumentProcessor(ABC):
    """Base class for document processors."""
    
    def __init__(self, chunk_size: int = 1000, chunk_overlap: int = 200):
        """Initialize the document processor.
        
        Args:
            chunk_size: The size of each text chunk
            chunk_overlap: The overlap between consecutive chunks
        """
        self.chunk_size = chunk_size
        self.chunk_overlap = chunk_overlap
    
    @abstractmethod
    def load_document(self, file_path: str) -> Document:
        """Load a document from a file.
        
        Args:
            file_path: Path to the document file
            
        Returns:
            Document: Loaded document
        """
        pass
    
    @abstractmethod
    def extract_text(self, content: bytes) -> str:
        """Extract text from document content.
        
        Args:
            content: Raw document content
            
        Returns:
            str: Extracted text
        """
        pass
    
    def create_chunks(self, text: str) -> List[DocumentChunk]:
        """Split text into overlapping chunks.
        
        Args:
            text: Text to split into chunks
            
        Returns:
            List[DocumentChunk]: List of text chunks
        """
        chunks = []
        start = 0
        chunk_index = 0
        
        while start < len(text):
            end = start + self.chunk_size
            chunk_text = text[start:end]
            
            if chunk_text:
                chunks.append(
                    DocumentChunk(
                        text=chunk_text,
                        chunk_index=chunk_index,
                    )
                )
            
            start = end - self.chunk_overlap
            chunk_index += 1
        
        return chunks
    
    @abstractmethod
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata:
        """Extract metadata from document.
        
        Args:
            file_path: Path to the document file
            content: Raw document content
            
        Returns:
            DocumentMetadata: Extracted metadata
        """
        pass 