"""
PDF document processor implementation.
"""
import io
import os
from datetime import datetime
from typing import Dict, Optional

from pypdf import PdfReader

from ..models.document import Document, DocumentMetadata, DocumentType
from .base import BaseDocumentProcessor


class PDFProcessor(BaseDocumentProcessor):
    """Processor for PDF documents."""
    
    def load_document(self, file_path: str) -> Document:
        """Load a PDF document from a file.
        
        Args:
            file_path: Path to the PDF file
            
        Returns:
            Document: Loaded document
        """
        with open(file_path, "rb") as file:
            content = file.read()
            text = self.extract_text(content)
            metadata = self.get_metadata(file_path, content)
            chunks = self.create_chunks(text)
            
            return Document(
                content=text,
                metadata=metadata,
                chunks=chunks
            )
    
    def extract_text(self, content: bytes) -> str:
        """Extract text content from PDF bytes.
        
        Args:
            content: PDF file content as bytes
            
        Returns:
            str: Extracted text content
        """
        pdf_file = io.BytesIO(content)
        reader = PdfReader(pdf_file)
        
        text_content = []
        for page in reader.pages:
            text_content.append(page.extract_text())
        
        return "\n".join(text_content)
    
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata:
        """Extract metadata from PDF document.
        
        Args:
            file_path: Path to the PDF file
            content: PDF file content as bytes
            
        Returns:
            DocumentMetadata: Extracted metadata
        """
        pdf_file = io.BytesIO(content)
        reader = PdfReader(pdf_file)
        
        # Extract basic metadata
        metadata = reader.metadata or {}
        
        # Clean up metadata keys and values
        custom_metadata: Dict[str, str] = {}
        for key, value in metadata.items():
            if key and value:
                # Remove leading slash from keys
                clean_key = key[1:] if key.startswith("/") else key
                custom_metadata[clean_key] = str(value)
        
        return DocumentMetadata(
            filename=os.path.basename(file_path),
            file_type=DocumentType.PDF,
            file_size=len(content),
            num_pages=len(reader.pages),
            author=custom_metadata.get("Author"),
            title=custom_metadata.get("Title"),
            custom_metadata=custom_metadata,
        ) 