"""
Logging utilities for the LLM Document Analysis package.
"""
import logging
import sys
from typing import Optional

from ..config.settings import settings


def setup_logging(
    level: Optional[str] = None,
    log_format: Optional[str] = None
) -> logging.Logger:
    """Set up logging configuration.
    
    Args:
        level: Log level (default: from settings)
        log_format: Log format string (default: standard format)
        
    Returns:
        logging.Logger: Configured logger instance
        
    Raises:
        ValueError: If an invalid log level is provided
    """
    level = level or settings.log_level
    log_format = log_format or "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    
    logger = logging.getLogger("llm_document_analysis")
    
    # Only add handler if logger doesn't have any
    if not logger.handlers:
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter(log_format)
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    else:
        # Update existing handler's formatter if format is provided
        if log_format:
            logger.handlers[0].setFormatter(logging.Formatter(log_format))
    
    try:
        logger.setLevel(level.upper())
    except (ValueError, AttributeError) as e:
        raise ValueError(f"Invalid log level: {level}") from e
    
    logger.propagate = False
    
    return logger


# Create default logger instance
logger = setup_logging() 