"""
Tests for the document analyzer module.
"""
from typing import Any, Dict, List
from unittest.mock import MagicMock, patch

import pytest
from langchain_ollama import OllamaLLM
from langchain.schema.language_model import BaseLanguageModel

from llm_document_analysis.config.settings import LLMProvider
from llm_document_analysis.core.analyzer import DocumentAnalyzer
from llm_document_analysis.models.document import Document, DocumentChunk, DocumentMetadata, DocumentType


@pytest.fixture
def mock_llm() -> MagicMock:
    """Create a mock LLM."""
    mock = MagicMock(spec=BaseLanguageModel)
    mock.invoke.return_value = "Mock LLM response"
    return mock


@pytest.fixture
def mock_ollama_llm() -> MagicMock:
    """Create a mock OllamaLLM instance."""
    mock = MagicMock(spec=OllamaLLM)
    mock.invoke.return_value = "Mock Ollama response"
    return mock


@pytest.fixture
def analyzer(mock_settings: Any, mock_ollama_llm: MagicMock) -> DocumentAnalyzer:
    """Create a document analyzer instance with mocked LLM."""
    with patch("llm_document_analysis.core.analyzer.OllamaLLM", return_value=mock_ollama_llm):
        return DocumentAnalyzer()


@pytest.fixture
def sample_document_with_chunks() -> Document:
    """Create a sample document with multiple chunks."""
    chunks = [
        DocumentChunk(text=f"Chunk {i} content", chunk_index=i)
        for i in range(3)
    ]
    return Document(
        content="Full document content",
        metadata=DocumentMetadata(
            filename="test.pdf",
            file_type=DocumentType.PDF,
            file_size=1000,
            num_pages=3,
            author="Test Author",
            title="Test Document",
        ),
        chunks=chunks,
    )


class TestDocumentAnalyzer:
    """Test cases for DocumentAnalyzer class."""
    
    def test_init_with_defaults(self, mock_settings: Any, mock_ollama_llm: MagicMock) -> None:
        """Test analyzer initialization with default values."""
        with patch("llm_document_analysis.core.analyzer.OllamaLLM", return_value=mock_ollama_llm):
            analyzer = DocumentAnalyzer()
            
            assert analyzer.model_name == mock_settings.model_name
            assert analyzer.temperature == mock_settings.temperature
            assert analyzer.max_tokens == mock_settings.max_tokens
            assert analyzer.llm_provider == mock_settings.llm_provider
            assert isinstance(analyzer.llm, MagicMock)
            assert analyzer.llm is mock_ollama_llm
    
    @patch.dict("os.environ", {"OPENAI_API_KEY": "test-key"})
    def test_init_with_custom_values(self, mock_settings: Any) -> None:
        """Test analyzer initialization with custom values."""
        custom_model = "custom-model"
        custom_temp = 0.8
        custom_tokens = 2000
        
        analyzer = DocumentAnalyzer(
            model_name=custom_model,
            temperature=custom_temp,
            max_tokens=custom_tokens,
            llm_provider=LLMProvider.OPENAI,
        )
        
        assert analyzer.model_name == custom_model
        assert analyzer.temperature == custom_temp
        assert analyzer.max_tokens == custom_tokens
        assert analyzer.llm_provider == LLMProvider.OPENAI

    def test_init_with_invalid_provider(self) -> None:
        """Test initialization with an invalid LLM provider."""
        with pytest.raises(ValueError):
            DocumentAnalyzer(llm_provider="invalid_provider")
    
    def test_analyze_document_full(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test analyzing a full document."""
        prompt = "Analyze this text: {text}"
        
        result = analyzer.analyze_document(sample_document, prompt)
        
        assert result == "Mock Ollama response"
        mock_ollama_llm.invoke.assert_called_once()
        call_args = mock_ollama_llm.invoke.call_args[0][0]
        assert sample_document.content in call_args.messages[0].content
    
    def test_analyze_document_chunks(
        self,
        analyzer: DocumentAnalyzer,
        sample_document_with_chunks: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test analyzing document chunks."""
        prompt = "Analyze this chunk: {text}"
        
        results = analyzer.analyze_document(sample_document_with_chunks, prompt, per_chunk=True)
        
        assert isinstance(results, list)
        assert len(results) == len(sample_document_with_chunks.chunks)
        assert all(result == "Mock Ollama response" for result in results)
        assert mock_ollama_llm.invoke.call_count == len(sample_document_with_chunks.chunks)

        # Verify each chunk was analyzed
        for i, chunk in enumerate(sample_document_with_chunks.chunks):
            call_args = mock_ollama_llm.invoke.call_args_list[i][0][0]
            assert chunk.text in call_args.messages[0].content
    
    def test_summarize(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test document summarization."""
        max_length = 500
        
        summary = analyzer.summarize(sample_document, max_length=max_length)
        
        assert summary == "Mock Ollama response"
        mock_ollama_llm.invoke.assert_called_once()
        call_args = mock_ollama_llm.invoke.call_args[0][0]
        prompt_content = call_args.messages[0].content
        assert str(max_length) in prompt_content
        assert sample_document.content in prompt_content

    def test_summarize_with_invalid_length(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
    ) -> None:
        """Test summarization with invalid max_length."""
        with pytest.raises(ValueError):
            analyzer.summarize(sample_document, max_length=0)
    
    def test_summarize_truncation(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test summary truncation when response exceeds max_length."""
        max_length = 10
        mock_ollama_llm.invoke.return_value = "This is a very long response that needs to be truncated"
        
        summary = analyzer.summarize(sample_document, max_length=max_length)
        
        assert len(summary) <= max_length
        assert summary == "This is a"  # Truncated at word boundary
        mock_ollama_llm.invoke.assert_called_once()

    def test_extract_key_information(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test extracting key information."""
        info_types = ["topics", "dates", "names"]
        
        result = analyzer.extract_key_information(sample_document, info_types)
        
        assert result == "Mock Ollama response"
        mock_ollama_llm.invoke.assert_called_once()
        call_args = mock_ollama_llm.invoke.call_args[0][0]
        prompt_content = call_args.messages[0].content
        assert all(info_type in prompt_content for info_type in info_types)
        assert sample_document.content in prompt_content

    def test_extract_key_information_empty_types(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
    ) -> None:
        """Test extracting key information with empty info types."""
        with pytest.raises(ValueError):
            analyzer.extract_key_information(sample_document, [])

    def test_document_model_methods(self, sample_document_with_chunks: Document) -> None:
        """Test Document model's to_dict method and num_chunks property."""
        # Test to_dict method
        doc_dict = sample_document_with_chunks.to_dict()
        assert isinstance(doc_dict, dict)
        assert doc_dict["content"] == sample_document_with_chunks.content
        assert doc_dict["metadata"]["filename"] == sample_document_with_chunks.metadata.filename
        assert len(doc_dict["chunks"]) == len(sample_document_with_chunks.chunks)
        
        # Test num_chunks property
        assert sample_document_with_chunks.num_chunks == len(sample_document_with_chunks.chunks)
        assert sample_document_with_chunks.num_chunks == 3  # Based on fixture setup 

    def test_analyze_document_string_response(
        self,
        analyzer: DocumentAnalyzer,
        sample_document: Document,
        mock_ollama_llm: MagicMock,
    ) -> None:
        """Test analyzing a document when LLM returns a string response."""
        # Configure mock to return a string instead of an object with content
        mock_ollama_llm.invoke.return_value = "Direct string response"
        
        prompt = "Analyze this text: {text}"
        result = analyzer.analyze_document(sample_document, prompt)
        
        assert result == "Direct string response"
        mock_ollama_llm.invoke.assert_called_once()
        call_args = mock_ollama_llm.invoke.call_args[0][0]
        assert sample_document.content in call_args.messages[0].content 

    def test_initialize_llm_invalid_provider(self, mock_settings: Any) -> None:
        """Test initialization with an invalid LLM provider."""
        analyzer = DocumentAnalyzer()
        analyzer.llm_provider = "invalid"  # Set invalid provider directly
        
        with pytest.raises(ValueError, match="Invalid LLM provider: invalid"):
            analyzer._initialize_llm() 