"""
Tests for the base document processor.
"""
import os
from pathlib import Path
from typing import List

import pytest
from abc import ABC

from llm_document_analysis.models.document import Document, DocumentChunk, DocumentMetadata, DocumentType
from llm_document_analysis.processors.base import BaseDocumentProcessor


# Move TestProcessor outside of test module scope to fix warning
class MinimalProcessor(BaseDocumentProcessor):
    """Minimal concrete implementation of BaseDocumentProcessor for testing."""
    def load_document(self, file_path: str) -> Document: pass
    def extract_text(self, content: bytes) -> str: pass
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata: pass


class ConcreteProcessor(BaseDocumentProcessor):
    """Full concrete implementation of BaseDocumentProcessor for testing."""
    
    def load_document(self, file_path: str) -> Document:
        """Test implementation."""
        return Document(
            content="test content",
            metadata=DocumentMetadata(
                filename="test.txt",
                file_type=DocumentType.TXT,
                file_size=100,
            )
        )
    
    def extract_text(self, content: bytes) -> str:
        """Test implementation."""
        return content.decode()
    
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata:
        """Test implementation."""
        return DocumentMetadata(
            filename="test.txt",
            file_type=DocumentType.TXT,
            file_size=len(content),
        )


class ConcreteTestProcessor(BaseDocumentProcessor):
    """Concrete implementation of BaseDocumentProcessor for testing."""
    
    def load_document(self, file_path: str) -> Document:
        """Test implementation of load_document."""
        with open(file_path, 'rb') as f:
            content = f.read()
        text = self.extract_text(content)
        metadata = self.get_metadata(file_path, content)
        chunks = self.create_chunks(text)
        return Document(content=text, metadata=metadata, chunks=chunks)
    
    def extract_text(self, content: bytes) -> str:
        """Test implementation of extract_text."""
        return content.decode('utf-8')
    
    def get_metadata(self, file_path: str, content: bytes) -> DocumentMetadata:
        """Test implementation of get_metadata."""
        return DocumentMetadata(
            filename=os.path.basename(file_path),
            file_type=DocumentType.TXT,
            file_size=len(content),
            num_pages=1
        )


@pytest.fixture
def test_processor() -> ConcreteTestProcessor:
    """Create a test processor instance."""
    return ConcreteTestProcessor(chunk_size=10, chunk_overlap=2)


@pytest.fixture
def test_file(tmp_path: Path) -> str:
    """Create a test file."""
    file_path = tmp_path / "test.txt"
    with open(file_path, 'w') as f:
        f.write("This is a test document for processing.")
    return str(file_path)


class TestBaseDocumentProcessor:
    """Test cases for BaseDocumentProcessor class."""
    
    def test_abstract_methods(self) -> None:
        """Test that abstract methods raise NotImplementedError."""
        # Verify that BaseDocumentProcessor is abstract
        assert issubclass(BaseDocumentProcessor, ABC)
        
        # Verify that abstract methods are defined
        processor = MinimalProcessor()
        assert hasattr(processor, 'load_document')
        assert hasattr(processor, 'extract_text')
        assert hasattr(processor, 'get_metadata')
    
    def test_create_chunks_empty_text(self) -> None:
        """Test chunk creation with empty text."""
        processor = ConcreteProcessor()
        chunks = processor.create_chunks("")
        assert len(chunks) == 0
    
    def test_create_chunks_short_text(self) -> None:
        """Test chunk creation with text shorter than chunk size."""
        processor = ConcreteProcessor()
        text = "Short test text"
        chunks = processor.create_chunks(text)
        assert len(chunks) == 1
        assert chunks[0].text == text
        assert chunks[0].chunk_index == 0
    
    def test_create_chunks_long_text(self) -> None:
        """Test chunk creation with text longer than chunk size."""
        processor = ConcreteProcessor(chunk_size=10, chunk_overlap=2)
        text = "This is a longer test text that should be split into multiple chunks"
        chunks = processor.create_chunks(text)
        
        assert len(chunks) > 1
        assert all(len(chunk.text) <= 10 for chunk in chunks)
        assert all(isinstance(chunk.chunk_index, int) for chunk in chunks)
        
        # Test overlap
        if len(chunks) > 1:
            first_chunk_end = chunks[0].text[-2:]  # overlap size is 2
            second_chunk_start = chunks[1].text[:2]
            assert first_chunk_end == second_chunk_start
    
    def test_create_chunks_with_empty_chunk(self) -> None:
        """Test handling of empty chunks during text splitting."""
        processor = ConcreteProcessor(chunk_size=5, chunk_overlap=2)
        text = "a b c"  # Will create empty chunks due to small size and overlap
        chunks = processor.create_chunks(text)
        assert all(len(chunk.text) > 0 for chunk in chunks)  # No empty chunks
    
    def test_create_chunks_exact_size(self) -> None:
        """Test chunk creation with text exactly matching chunk size."""
        processor = ConcreteProcessor(chunk_size=5, chunk_overlap=2)
        text = "12345"
        chunks = processor.create_chunks(text)
        
        # Due to overlap, we get two chunks:
        # 1. "12345" (first chunk)
        # 2. "45" (second chunk due to overlap)
        assert len(chunks) == 2
        assert chunks[0].text == "12345"
        assert chunks[1].text == "45"
        assert chunks[0].chunk_index == 0
        assert chunks[1].chunk_index == 1
        
        # Test with no overlap
        processor = ConcreteProcessor(chunk_size=5, chunk_overlap=0)
        chunks = processor.create_chunks(text)
        assert len(chunks) == 1
        assert chunks[0].text == text
    
    def test_processor_initialization(self) -> None:
        """Test processor initialization with custom values."""
        processor = ConcreteProcessor(chunk_size=500, chunk_overlap=100)
        assert processor.chunk_size == 500
        assert processor.chunk_overlap == 100
    
    def test_load_document(self, test_processor: ConcreteTestProcessor, test_file: str) -> None:
        """Test loading a document."""
        doc = test_processor.load_document(test_file)
        
        assert isinstance(doc, Document)
        assert doc.content == "This is a test document for processing."
        assert doc.metadata.filename == "test.txt"
        assert doc.metadata.file_type == DocumentType.TXT
        assert len(doc.chunks) > 0
    
    def test_extract_text(self, test_processor: ConcreteTestProcessor) -> None:
        """Test text extraction."""
        content = b"Test content"
        text = test_processor.extract_text(content)
        
        assert text == "Test content"
    
    def test_create_chunks(self, test_processor: ConcreteTestProcessor) -> None:
        """Test creating text chunks."""
        text = "This is a test document"
        chunks = test_processor.create_chunks(text)
        
        assert isinstance(chunks, list)
        assert all(isinstance(chunk, DocumentChunk) for chunk in chunks)
        assert len(chunks) > 0
        
        # Test chunk size and overlap
        assert len(chunks[0].text) <= test_processor.chunk_size
        if len(chunks) > 1:
            overlap = len(
                set(chunks[0].text[-test_processor.chunk_overlap:]) &
                set(chunks[1].text[:test_processor.chunk_overlap])
            )
            assert overlap > 0
    
    def test_get_metadata(self, test_processor: ConcreteTestProcessor, test_file: str) -> None:
        """Test metadata extraction."""
        with open(test_file, 'rb') as f:
            content = f.read()
        
        metadata = test_processor.get_metadata(test_file, content)
        
        assert isinstance(metadata, DocumentMetadata)
        assert metadata.filename == "test.txt"
        assert metadata.file_type == DocumentType.TXT
        assert metadata.file_size == len(content)
        assert metadata.num_pages == 1 