"""
Integration tests for the LLM Document Analysis package.
"""
import os
from pathlib import Path

import pytest
from langchain_ollama import OllamaLLM

from llm_document_analysis import DocumentAnalyzer
from llm_document_analysis.processors.pdf import PDFProcessor


@pytest.fixture
def test_pdf_path(test_data_dir: Path) -> Path:
    """Get the path to the test PDF file."""
    return test_data_dir / "sample.pdf"


@pytest.fixture(autouse=True)
def setup_ollama_env():
    """Setup Ollama environment variables."""
    os.environ["OLLAMA_BASE_URL"] = "http://localhost:11434"
    os.environ["OLLAMA_MODEL"] = "deepseek-r1:32b"
    os.environ["MODEL_NAME"] = "deepseek-r1:32b"


@pytest.mark.integration
class TestOllamaIntegration:
    """Integration tests for Ollama-based document analysis."""

    def test_ollama_connection(self):
        """Test that we can connect to Ollama."""
        llm = OllamaLLM(
            model="deepseek-r1:32b",
            base_url="http://localhost:11434"
        )
        response = llm.invoke("Test connection")
        assert isinstance(response, str)
        assert len(response) > 0

    def test_full_document_analysis_flow(self, test_pdf_path: Path):
        """Test the complete document analysis flow with Ollama."""
        # Initialize components
        processor = PDFProcessor()
        analyzer = DocumentAnalyzer(model_name="deepseek-r1:32b")

        # Load and process document
        document = processor.load_document(str(test_pdf_path))
        assert document.content
        assert document.metadata.file_type == "pdf"

        # Test summarization
        summary = analyzer.summarize(document, max_length=200)
        assert isinstance(summary, str)
        assert len(summary) > 0
        assert len(summary) <= 200

        # Test key information extraction
        info_types = ["main_topics", "key_points"]
        info = analyzer.extract_key_information(document, info_types)
        assert isinstance(info, str)
        assert len(info) > 0

        # Test custom analysis
        custom_prompt = """
        Analyze this text and identify:
        1. Main ideas
        2. Supporting details
        3. Conclusions

        Text: {text}
        """
        analysis = analyzer.analyze_document(document, custom_prompt)
        assert isinstance(analysis, str)
        assert len(analysis) > 0

        # Test chunk-based analysis
        chunk_results = analyzer.analyze_document(
            document,
            "Summarize this chunk: {text}",
            per_chunk=True
        )
        assert isinstance(chunk_results, list)
        assert len(chunk_results) == len(document.chunks) 