"""
Tests for the logging utilities.
"""
import logging
import sys
from unittest.mock import MagicMock, patch

import pytest

from llm_document_analysis.utils.logging import setup_logging


@pytest.fixture
def mock_logger() -> MagicMock:
    """Create a mock logger."""
    return MagicMock(spec=logging.Logger)


@pytest.fixture
def mock_handler() -> MagicMock:
    """Create a mock stream handler."""
    return MagicMock(spec=logging.StreamHandler)


@pytest.fixture(autouse=True)
def clean_logger() -> None:
    """Clean up logger between tests."""
    logger = logging.getLogger("llm_document_analysis")
    if logger.handlers:
        logger.handlers.clear()
    yield


class TestLogging:
    """Test cases for logging utilities."""
    
    def test_setup_logging_with_defaults(self, mock_settings: MagicMock) -> None:
        """Test logging setup with default values."""
        logger = setup_logging()
        
        assert isinstance(logger, logging.Logger)
        assert logger.level == logging.INFO
        assert len(logger.handlers) == 1
        assert isinstance(logger.handlers[0], logging.StreamHandler)
        assert not logger.propagate
    
    def test_setup_logging_with_custom_values(self) -> None:
        """Test logging setup with custom values."""
        custom_level = "DEBUG"
        custom_format = "%(levelname)s - %(message)s"
        
        logger = setup_logging(level=custom_level, log_format=custom_format)
        
        assert isinstance(logger, logging.Logger)
        assert logger.level == logging.DEBUG
        assert len(logger.handlers) == 1
        handler = logger.handlers[0]
        assert isinstance(handler, logging.StreamHandler)
        assert handler.formatter._fmt == custom_format
        assert not logger.propagate
    
    def test_setup_logging_reuse_existing_logger(self) -> None:
        """Test that setup_logging reuses existing logger."""
        # First call to setup_logging
        logger1 = setup_logging()
        handlers_count = len(logger1.handlers)
        
        # Second call to setup_logging with different format
        custom_format = "%(levelname)s - %(message)s"
        logger2 = setup_logging(log_format=custom_format)
        
        # Verify it's the same logger instance and format is updated
        assert logger1 is logger2
        assert len(logger2.handlers) == handlers_count
        assert logger2.handlers[0].formatter._fmt == custom_format
    
    def test_setup_logging_invalid_level(self) -> None:
        """Test logging setup with invalid log level."""
        with pytest.raises(ValueError, match="Invalid log level: INVALID_LEVEL"):
            setup_logging(level="INVALID_LEVEL")
    
    def test_setup_logging_stdout_handler(self) -> None:
        """Test that logging handler uses stdout."""
        logger = setup_logging()
        
        handler = logger.handlers[0]
        assert handler.stream is sys.stdout
    
    def test_setup_logging_none_level(self) -> None:
        """Test logging setup with None level."""
        logger = setup_logging(level=None)
        assert logger.level == logging.INFO
    
    def test_setup_logging_none_format(self) -> None:
        """Test logging setup with None format."""
        default_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        logger = setup_logging(log_format=None)
        assert logger.handlers[0].formatter._fmt == default_format
    
    def test_setup_logging_update_format(self) -> None:
        """Test updating format of existing logger."""
        # Create logger with default format
        logger1 = setup_logging()
        original_handler = logger1.handlers[0]
        
        # Update format
        new_format = "%(levelname)s: %(message)s"
        logger2 = setup_logging(log_format=new_format)
        
        # Verify handler is reused but format is updated
        assert logger2.handlers[0] is original_handler
        assert logger2.handlers[0].formatter._fmt == new_format 