"""
Tests for document processors.
"""
import io
from datetime import datetime
from pathlib import Path
from unittest.mock import MagicMock, mock_open, patch, PropertyMock

import pytest
from pypdf import PdfReader

from llm_document_analysis.models.document import Document, DocumentMetadata, DocumentType
from llm_document_analysis.processors.pdf import PDFProcessor


@pytest.fixture
def pdf_processor() -> PDFProcessor:
    """Create a PDF processor instance."""
    return PDFProcessor()


@pytest.fixture
def mock_pdf_file() -> bytes:
    """Create a mock PDF file content."""
    return b"""%PDF-1.4
1 0 obj
<</Type /Catalog
/Pages 2 0 R>>
endobj
2 0 obj
<</Type /Pages
/Kids [3 0 R]
/Count 1>>
endobj
3 0 obj
<</Type /Page
/Parent 2 0 R
/Resources <</Font <</F1 4 0 R>>>>
/MediaBox [0 0 612 792]
/Contents 5 0 R>>
endobj
4 0 obj
<</Type /Font
/Subtype /Type1
/BaseFont /Helvetica>>
endobj
5 0 obj
<</Length 44>>
stream
BT
/F1 12 Tf
72 720 Td
(Sample PDF for testing) Tj
ET
endstream
endobj
xref
0 6
0000000000 65535 f
0000000009 00000 n
0000000056 00000 n
0000000111 00000 n
0000000209 00000 n
0000000274 00000 n
trailer
<</Size 6
/Root 1 0 R>>
startxref
364
%%EOF"""


@pytest.fixture
def mock_pdf_reader() -> MagicMock:
    """Create a mock pypdf.PdfReader instance."""
    mock = MagicMock(spec=PdfReader)
    mock.pages = [MagicMock()]
    mock.pages[0].extract_text.return_value = "Sample PDF for testing"
    mock.metadata = {
        "/Author": "Test Author",
        "/Title": "Test Document",
        "/Keywords": "test, pdf, document",
    }
    return mock


class TestPDFProcessor:
    """Test cases for PDFProcessor class."""
    
    def test_init_with_defaults(self) -> None:
        """Test processor initialization with default values."""
        processor = PDFProcessor()
        assert processor.chunk_size == 1000
        assert processor.chunk_overlap == 200
    
    def test_init_with_custom_values(self) -> None:
        """Test processor initialization with custom values."""
        processor = PDFProcessor(chunk_size=500, chunk_overlap=100)
        assert processor.chunk_size == 500
        assert processor.chunk_overlap == 100
    
    @patch("builtins.open", new_callable=mock_open)
    @patch("llm_document_analysis.processors.pdf.PdfReader")
    @patch("os.path.getmtime")
    def test_load_document(
        self,
        mock_getmtime: MagicMock,
        mock_pdf_reader_class: MagicMock,
        mock_file: MagicMock,
        pdf_processor: PDFProcessor,
        mock_pdf_reader: MagicMock,
        mock_pdf_file: bytes,
        tmp_path: Path,
    ) -> None:
        """Test loading a PDF document."""
        # Setup
        pdf_path = tmp_path / "test.pdf"
        mock_file.return_value.read.return_value = mock_pdf_file
        mock_pdf_reader_class.return_value = mock_pdf_reader
        mock_getmtime.return_value = datetime.now().timestamp()

        # Configure mock PDF reader
        mock_pdf_reader.metadata = None  # Test with no metadata
        mock_page = MagicMock()
        mock_page.extract_text.return_value = "Sample PDF for testing"
        mock_pdf_reader.pages = [mock_page]

        # Execute
        document = pdf_processor.load_document(str(pdf_path))
        
        # Verify
        assert isinstance(document, Document)
        assert document.content == "Sample PDF for testing"
        assert document.metadata.filename == "test.pdf"
        assert document.metadata.file_type == DocumentType.PDF
        assert document.metadata.author is None  # No metadata available
        assert document.metadata.title is None  # No metadata available

    def test_load_document_with_invalid_path(self, pdf_processor: PDFProcessor) -> None:
        """Test loading a PDF document with an invalid file path."""
        with pytest.raises(FileNotFoundError):
            pdf_processor.load_document("nonexistent.pdf")
    
    @patch("llm_document_analysis.processors.pdf.PdfReader")
    def test_extract_text(
        self,
        mock_pdf_reader_class: MagicMock,
        pdf_processor: PDFProcessor,
        mock_pdf_reader: MagicMock,
        mock_pdf_file: bytes,
    ) -> None:
        """Test extracting text from PDF content."""
        # Setup
        mock_page = MagicMock()
        mock_page.extract_text.return_value = "Sample PDF for testing"
        mock_pdf_reader.pages = [mock_page]
        mock_pdf_reader_class.return_value = mock_pdf_reader

        # Execute
        text = pdf_processor.extract_text(mock_pdf_file)
        
        # Verify
        assert text == "Sample PDF for testing"
        mock_pdf_reader_class.assert_called_once()
        assert mock_page.extract_text.call_count == 1

    def test_extract_text_with_multiple_pages(
        self,
        pdf_processor: PDFProcessor,
        mock_pdf_reader: MagicMock,
        mock_pdf_file: bytes,
    ) -> None:
        """Test extracting text from a multi-page PDF."""
        with patch("llm_document_analysis.processors.pdf.PdfReader") as mock_reader_class:
            # Setup multiple pages
            mock_pages = [MagicMock() for _ in range(3)]
            for i, page in enumerate(mock_pages):
                page.extract_text.return_value = f"Page {i+1} content"
            mock_reader = MagicMock()
            mock_reader.pages = mock_pages
            mock_reader_class.return_value = mock_reader

            # Execute
            text = pdf_processor.extract_text(mock_pdf_file)

            # Verify
            assert "Page 1 content" in text
            assert "Page 2 content" in text
            assert "Page 3 content" in text
            assert text == "Page 1 content\nPage 2 content\nPage 3 content"
    
    @patch("llm_document_analysis.processors.pdf.PdfReader")
    @patch("os.path.getmtime")
    def test_get_metadata(
        self,
        mock_getmtime: MagicMock,
        mock_pdf_reader_class: MagicMock,
        pdf_processor: PDFProcessor,
        mock_pdf_reader: MagicMock,
        mock_pdf_file: bytes,
        tmp_path: Path,
    ) -> None:
        """Test extracting metadata from PDF document."""
        # Setup
        pdf_path = tmp_path / "test.pdf"
        mock_pdf_reader_class.return_value = mock_pdf_reader
        mock_getmtime.return_value = datetime.now().timestamp()

        # Test with full metadata
        mock_pdf_reader.metadata = {
            "/Author": "Test Author",
            "/Title": "Test Title",
            "/Keywords": "test, pdf",
            "/Creator": "Test Creator",
            "/Producer": "Test Producer"
        }
        mock_page = MagicMock()
        mock_pdf_reader.pages = [mock_page]

        # Execute
        metadata = pdf_processor.get_metadata(str(pdf_path), mock_pdf_file)
        
        # Verify
        assert isinstance(metadata, DocumentMetadata)
        assert metadata.filename == "test.pdf"
        assert metadata.file_type == DocumentType.PDF
        assert metadata.file_size == len(mock_pdf_file)
        assert metadata.num_pages == 1
        assert metadata.author == "Test Author"
        assert metadata.title == "Test Title"
        assert metadata.custom_metadata["Keywords"] == "test, pdf"
        assert metadata.custom_metadata["Creator"] == "Test Creator"
        assert metadata.custom_metadata["Producer"] == "Test Producer"

    def test_get_metadata_with_missing_fields(
        self,
        pdf_processor: PDFProcessor,
        mock_pdf_reader: MagicMock,
        mock_pdf_file: bytes,
        tmp_path: Path,
    ) -> None:
        """Test metadata extraction with missing fields."""
        with patch("llm_document_analysis.processors.pdf.PdfReader") as mock_reader_class:
            # Setup
            pdf_path = tmp_path / "test.pdf"
            mock_reader = MagicMock()
            mock_reader.metadata = {}  # Empty metadata
            mock_reader.pages = [MagicMock()]
            mock_reader_class.return_value = mock_reader

            # Execute
            metadata = pdf_processor.get_metadata(str(pdf_path), mock_pdf_file)

            # Verify
            assert metadata.author is None
            assert metadata.title is None
            assert metadata.custom_metadata == {}
    
    def test_create_chunks(self, pdf_processor: PDFProcessor) -> None:
        """Test creating text chunks."""
        # Test with short text (single chunk)
        short_text = "This is a short test document."
        short_chunks = pdf_processor.create_chunks(short_text)
        assert len(short_chunks) == 1
        assert short_chunks[0].text == short_text
        assert short_chunks[0].chunk_index == 0

        # Test with long text (multiple chunks)
        long_text = "This is a test document. " * 100
        long_chunks = pdf_processor.create_chunks(long_text)
        assert len(long_chunks) > 1
        assert all(len(chunk.text) <= pdf_processor.chunk_size for chunk in long_chunks)
        assert all(isinstance(chunk.chunk_index, int) for chunk in long_chunks)

        # Test chunk overlap
        if len(long_chunks) > 1:
            first_chunk_end = long_chunks[0].text[-pdf_processor.chunk_overlap:]
            second_chunk_start = long_chunks[1].text[:pdf_processor.chunk_overlap]
            assert first_chunk_end in second_chunk_start

        # Test with empty text
        empty_chunks = pdf_processor.create_chunks("")
        assert len(empty_chunks) == 0 