"""
Tests for the configuration settings.
"""
from unittest.mock import patch

import pytest

from llm_document_analysis.config.settings import LLMProvider, Settings


class TestSettings:
    """Test cases for Settings class."""
    
    def test_default_settings(self) -> None:
        """Test settings initialization with default values."""
        settings = Settings()
        
        assert settings.llm_provider == LLMProvider.OLLAMA
        assert settings.ollama_base_url == "http://localhost:11434"
        assert settings.ollama_model == "deepseek-coder:14b"
        assert settings.openai_api_key is None
        assert settings.model_name == "deepseek-coder:14b"
        assert settings.temperature == 0.0
        assert settings.max_tokens == 1000
        assert settings.chunk_size == 1000
        assert settings.chunk_overlap == 200
        assert settings.max_doc_size_mb == 10.0
        assert settings.log_level == "INFO"
    
    def test_settings_from_env(self) -> None:
        """Test settings initialization from environment variables."""
        env_vars = {
            "LLM_PROVIDER": "openai",
            "OLLAMA_BASE_URL": "http://custom:11434",
            "OLLAMA_MODEL": "custom-model",
            "OPENAI_API_KEY": "test-key",
            "MODEL_NAME": "gpt-4",
            "TEMPERATURE": "0.8",
            "MAX_TOKENS": "2000",
            "CHUNK_SIZE": "500",
            "CHUNK_OVERLAP": "100",
            "MAX_DOC_SIZE_MB": "20.0",
            "LOG_LEVEL": "DEBUG",
        }
        
        with patch.dict("os.environ", env_vars):
            settings = Settings()
            
            assert settings.llm_provider == LLMProvider.OPENAI
            assert settings.ollama_base_url == "http://custom:11434"
            assert settings.ollama_model == "custom-model"
            assert settings.openai_api_key == "test-key"
            assert settings.model_name == "gpt-4"
            assert settings.temperature == 0.8
            assert settings.max_tokens == 2000
            assert settings.chunk_size == 500
            assert settings.chunk_overlap == 100
            assert settings.max_doc_size_mb == 20.0
            assert settings.log_level == "DEBUG"
    
    def test_is_ollama_property(self) -> None:
        """Test is_ollama property."""
        settings = Settings(llm_provider=LLMProvider.OLLAMA)
        assert settings.is_ollama is True
        
        settings = Settings(llm_provider=LLMProvider.OPENAI)
        assert settings.is_ollama is False
    
    def test_is_openai_property(self) -> None:
        """Test is_openai property."""
        settings = Settings(llm_provider=LLMProvider.OPENAI)
        assert settings.is_openai is True
        
        settings = Settings(llm_provider=LLMProvider.OLLAMA)
        assert settings.is_openai is False
    
    def test_invalid_llm_provider(self) -> None:
        """Test settings with invalid LLM provider."""
        with pytest.raises(ValueError):
            Settings(llm_provider="invalid") 